pub struct LifecycleId(/* private fields */);Expand description
An identifier that can be set on sent messages, and picked by the sending client. If set,
lifecycle events will be generated, and eventually SocketEvent::OnLifecycleEnd will be
generated to indicate that the lifecycle isn’t tracked any longer. The value zero (0) is not a
valid lifecycle identifier, and will be interpreted as not having it set.
Implementations
Sourceimpl LifecycleId
impl LifecycleId
Sourcepub fn new(n: u64) -> Option<LifecycleId>
pub fn new(n: u64) -> Option<LifecycleId>
Creates a new LifecycleId.
Returns None if the value is zero, as zero is not a valid lifecycle identifier.
Sourcepub fn from(n: u64) -> LifecycleId
pub fn from(n: u64) -> LifecycleId